<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->
 
<h4>How Do I...Load and use the XmlDocument (W3C DOM)?</h4>

<div class="indent" style="width:660">
This sample illustrates the classes which implement the
<A href="http://www.w3.org/TR/REC-DOM-Level-1/level-one-core.html">W3C Document object Model (DOM) level 1 Core</A></A> and the
<A href="http://www.w3.org/TR/DOM-Level-2/core.html">Core DOM Level 2</A></A>. The DOM is an in-memory (cache) tree representation of an XML document and
enables the navigation and editing of this document.</div>

<h4>Loading and using the XmlDocument</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/DOMInterfaceXmlDocument/DOMInterfaceXmlDocument.src"
RunSample="/quickstart/howto/samples/Xml/DOMInterfaceXmlDocument/DOMInterfaceXmlDocument.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="DOMInterfaceXmlDocument.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
There are a number of classes implemented as part of the W3C DOM. Of these the <b>XmlNode</b> provides methods and properties to manipulate a node. Many of the other W3C classes
are specializations of the XmlNode class. The <b>XmlDocument</b> class represents the XML document and has a <b>Load</b> method to load the document from a file,
stream or an XmlReader. The list of classes is given in the table below.</div>

<br clear="left"><br>
<DIV class=indent>
<TABLE class=table 
style="border-style: solid" 
width="350">
<TBODY>
<TR>
<TH>Class</TH>
<TH>Derived from</TH>
</TR>
  <tr>
    <td width="179" height="19"><font size="1">XmlNode</font></td>
    <td width="308" height="19"><font size="1">implements ICloneable, IEnumerable</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlDocument</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlElement</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlAttribute</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlCharacterData</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlText</font></td>
    <td width="308" height="19"><font size="1">XmlCharacterData</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlComment</font></td>
    <td width="308" height="19"><font size="1">XmlCharacterData</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlCDataSection</font></td>
    <td width="308" height="19"><font size="1">XmlCharacterData</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlProcessingInstruction</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlNodeList</font></td>
    <td width="308" height="19"><font size="1">IEnumerable</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlNamedNodeMap</font></td>
    <td width="308" height="19"><font size="1">IEnumerable</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlAttributeCollection</font></td>
    <td width="308" height="19"><font size="1">XmlNamedNodeMap</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlDocumentFragment</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlDocumentType</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlEntityReference</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlEntity</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlNotation</font></td>
    <td width="308" height="19"><font size="1">XmlNode</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">XmlImplementation</font></td>
    <td width="308" height="19"><font size="1">Object</font></td>
  </tr>
  </TBODY></TABLE></DIV>
 
<br clear="left"><br>
<div class="indent" style="width:660">
The code illustrated below shows an XmlDocument being loaded from an XmlTextReader, and the contents of the document are then displayed.</div>

<div class="code">
<xmp>
XmlTextReader reader = null;

try
{
    // Load the XML from file
    Console.WriteLine ("Reading file {0} ...", m_Document);
    reader = new XmlTextReader (m_Document);
    Console.WriteLine ("File {0} read sucessfully ...", m_Document);

    // Create and XmlDocument from the XmlTextReader
    XmlDocument xmldocument = new XmlDocument();
    xmldocument.Load (reader);
    Console.WriteLine ("XmlDocument loaded with XML data successfully ...");

    // Process the supplied XML file
    Console.WriteLine ("Processing ...");
    Console.WriteLine ();

    // Start from the document Element
    DisplayTree(xmldocument.DocumentElement);
}
catch (Exception e)
{
    Console.WriteLine ("Exception: {0}", e.ToString());
}

finally
{
    if (reader != null)
        reader.Close();
}
</xmp></div>

<div class="indent" style="width:660">
To navigate the document, the DisplayTree method recursively iterates over the XmlDocument using the <b>HasChildNodes</b> and <b>FirstChild</b> properties
to move down the tree.
The <b>NextSibling</b> property moves to the node immediately next to the current node, returning null if there is no node to move to.
</div>

<div class="code">
<xmp>
public void DisplayTree(XmlNode node)
{       
    if (node != null)
        Format (node);

    if (node.HasChildNodes)
    {
        node = node.FirstChild;
        while (node != null)
        {
            DisplayTree(node);
            node = node.NextSibling;
        }
    }
}
</xmp></div>

<div class="indent" style="width:660">
The <b>Name</b> property gives the name of the node, and the <b>Value</b> property its value depending on the type. 
The table below shows the value returned for a given node type.</div>

<br clear="left"><br>
<DIV class=indent>
<TABLE class=table 
style="border-style: solid" 
width="350">
<TBODY>
<TR>
<TH>Type</TH>
<TH>Value</TH>
</TR>
  <tr>
    <td width="179" height="19"><font size="1">Attribute</font></td>
    <td width="308" height="19"><font size="1">Value of attribute</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">CDATA</font></td>
    <td width="308" height="19"><font size="1">Content of the CDATA Section</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Comment</font></td>
    <td width="308" height="19"><font size="1">Content of the comment</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Document</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">DocumentFragment</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">DocumentType</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Element</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Entity</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">EntityReference</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Notation</font></td>
    <td width="308" height="19"><font size="1">null</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">ProcessingInstruction</font></td>
    <td width="308" height="19">	<font size="1">Entire content excluding the target</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Text</font></td>
    <td width="308" height="19"><font size="1">Content of the text node</font></td>
  </tr>
  </TBODY></TABLE></DIV>
  
<br clear="left"><br><div class="indent" style="width:660">
The Format method, shown below, display the details of the current node using the Name and Value properties. For <b>XmlElement</b> node types the <b>Attributes</b>
property provides a list of attributes as a <b>XmlNamedNodeMap</b> class. This class contains <b>GetNamedItem</b> to retrieve an XmlNode specified by name and 
<b>SetNamedItem</b> to add an XmlNode using it's Name property. It also implements IEnumerable in order to support the 'foreach' statement.</div>

<div class="code">
<xmp>
// Format the output
private void Format (XmlNode node)
{
    if (!node.HasChildNodes)
    {
        Console.Write("\t" + node.Name + "<" + node.Value + ">\r\n");
    }
    else
    {
        Console.Write(node.Name);
        if(XmlNodeType.Element == node.NodeType)
        {
            XmlNamedNodeMap map = node.Attributes;
            foreach (XmlNode attrnode in map)
                Console.Write(" " + attrnode.Name + "<" + attrnode.Value  + "> ");
        }
        Console.WriteLine();
    }
}
</xmp></div>

<div class="indent" style="width:660">
The XmlNode class also has a <b>NodeType</b> property. See <A target=content href="ReadXMLFile.aspx">How Do I...Read XML from a file?</A> for a table of
the different node types. The output from running this sample is shown below.
</div>

<div class="code">
<xmp>
Reading file books.xml ...
File books.xml read sucessfully ...
XmlDocument loaded with XML data successfully ...
Processing ...

bookstore
book genre<autobiography>  publicationdate<1981>  ISBN<1-861003-11-0>
title
        #text<The Autobiography of Benjamin Franklin>
author
first-name
        #text<Benjamin>
last-name
        #text<Franklin>
price
        #text<8.99>
book genre<novel>  publicationdate<1967>  ISBN<0-201-63361-2>
title
        #text<The Confidence Man>
author
first-name
        #text<Herman>
last-name
        #text<Melville>
price
        #text<11.99>
book genre<philosophy>  publicationdate<1991>  ISBN<1-861001-57-6>
title
        #text<The Gorgias>
author
name
        #text<Plato>
price
        #text<9.99>
</xmp></div>
                                           
<H4>Summary</H4>
<OL>
<LI>The XmlDocument, XmlNode and other classes implement the <A href="http://www.w3.org/TR/REC-DOM-Level-1/level-one-core.html">W3C Document object Model level 1 Core</A></A>
and the <A href="http://www.w3.org/TR/DOM-Level-2/core.html">Core DOM Level 2</A></A> specifications.
<LI>The XmlDocument is an in-memory (cache) tree representation of an XML document.
<LI>There are different node types specializing from XmlNode to enable you to manipulate the XML document.
<LI>You can get faster, non-cached, forward only stream access to XML using the XmlTextReader and XmlTextWriter classes.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->







































