<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Load a DataSet with XML?</h4>

<div class="indent" style="width:660">
This sample illustrates how to load a DataSet with XML Data. The sample builds on the previous topic
<a href="/quickstart/howto/doc/Xml/DataSetMapXSDSchema.aspx">How Do I...Create DataSet mappings from an XSD schema?</a> by first loading XML data into
an XmlDataDocument and then accessing this data from the DataSet. The DataSet has previously loaded a schema in order to create the internal mappings.
This sample in effect shows the transition between XML data and the creation of relational objects to access that XML data.</div>

<h4>Loading a DataSet with XML</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/LoadDataSetXMLData/LoadDataSetXMLData.src"
RunSample="/quickstart/howto/samples/Xml/LoadDataSetXMLData/LoadDataSetXMLData.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="LoadDataSetXMLData.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
First the XSD schema <a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/LoadDataSetXMLData/&file=books.xsd">books.xsd</a>
is loaded into the <b>DataSet</b> property on the XmlDataDocument and the XML file
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/LoadDataSetXMLData/&file=books.xml">books.xml</a> is loaded with 
the XmlDataDocument <b>Load</b> method. This is shown in the code below.</div>

<div class="code"><xmp>
private const String m_Document = "books.xml";
private const String m_Schema = "books.xsd";

XmlDataDocument m_datadoc = new XmlDataDocument();
ParseSchema(m_Schema);
DisplayTableStructure();

m_datadoc.Load(m_Document);

DisplayTables(m_datadoc.DataSet);
</xmp></div>

<div class="indent" style="width:660">
As in the previous topic, by simply iterating over the collections of Tables, Columns and Rows and formatting the output we can display both the internal
table structure with the DisplayTableStructure() method (built with the
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/LoadDataSetXMLData/&file=books.xsd">books.xsd</a> schema file)
and the content of the XML file with the DisplayTables() method. This time we use the foreach keyword instead of a for loop to illustrate an alternative
mechanism for iterating over the collections. See <a href="/quickstart/howto/doc/Xml/DataSetMapXSDSchema.aspx">How Do I...Create DataSet mappings from an XSD schema?</a>
</div>

<div class="code"><xmp>
// Displays the contents of the DataSet tables
private void DisplayTables(DataSet dataset)
{
    // Navigate Dataset
    Console.WriteLine("Content of Tables ...\r\n");

    foreach(DataTable table in dataset.Tables)
    {
        Console.WriteLine("TableName = " + table.TableName);
        Console.WriteLine ("{0}", "---------");
        Console.WriteLine("Columns ...\r\n");

        foreach(DataColumn column in table.Columns)
        {
            Console.Write("{0,-22}",column.ColumnName);
        }
        Console.WriteLine("");
        Console.WriteLine("\r\nNumber of rows = {0}", table.Rows.All.Length);
        Console.WriteLine("Rows ...\r\n");

        foreach(DataRow row in table.Rows)
        {
            foreach(Object value in row.ItemArray)
            {
                Console.Write("{0,-22}",value.ToString());
            }
            Console.WriteLine();
        }
        Console.WriteLine();
    }
}
</xmp></div>

<div class="indent" style="width:660">
The output from DisplayTables() for the books.xml is illustrated below showing the table and column names and the row contents.
</div>

<div class="code"><xmp>
Content of Tables ...

TableName = book
---------
Columns ...

genre                 publicationdate       ISBN                  title                 book_Id               price

Number of rows = 3
Rows ...

autobiography         1981                  1-861003-11-0         The Autobiography of Benjamin Franklin 0    8.99
novel                 1967                  0-201-63361-2         The Confidence Man    1                     11.99
philosophy            1991                  1-861001-57-6         The Gorgias           2                     9.99

TableName = author
---------
Columns ...

first-name            last-name             book_Id

Number of rows = 3
Rows ...

Benjamin              Franklin              0
Herman                Melville              1
Sidas                 Plato                 2
</xmp></div>

<H4>Summary</H4>
<OL>
<LI>XML data which has been loaded into an XmlDataDocument can be accessed via the relational methods on the DataSet property.
<LI>XML data can also be read when relational data has been entered via the XmlDataDocument, DataSet property.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->

























