<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Load XML data into an XmlDataDocument?</h4>

<div class="indent" style="width:660">
This sample illustrates how to load XML into the XmlDataDocument class. This class extends the XmlDocument class, allowing structured data to be stored, 
retrieved, and manipulated through a relational DataSet. Also see the <a href="XmlDataSet.aspx">How Do I...Use XML and the DataSet class?</a> topic.
</A></div>

<h4>Loading XML data into an XmlDataDocument</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/LoadXmlDocument/LoadXmlDocument.src"
RunSample="/quickstart/howto/samples/Xml/LoadXmlDocument/LoadXmlDocument.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="LoadXmlDocument.aspx"
runat="server" />

<br clear="left"><br><div class="indent" style="width:660">
The XmlDataDocument class provides an XML API onto loaded XML data by extending the XmlDocument (W3C DOM) class.
(See <a target=content href="DOMInterfaceXmlDocument.aspx">How Do I...Load and use the XmlDocument (W3C DOM)?</a>) and has a close affiliation with the DataSet
class, which provides a relational view onto the loaded XML data.
This combination allows for components to mix XML and relational views of the same underlying data, irrespective of the source of the
data. As detailed in later topics, there are differences between these views. Most notable is that XML data that does not map to the relational view cannot
be accessed via the relational objects (Tables and Rows). It is however, still possible to easily move from a relational row to the equivalent XML element
and hence access this XML data.</div>
 
<br clear="left"><div class="indent" style="width:660">
The XmlDataDocument can be used anywhere an XmlDocument is used; all topics that describe the XmlDocument apply equally to the XmlDataDocument.</div>

<br clear="left"><div class="indent" style="width:660">
The following topics are related to this topic.</div>

<OL>
<LI><A target=content href="DOMInterfaceXmlDocument.aspx">How Do I...Load and use the XmlDocument (W3C DOM)?</A>
<br clear="left"><br>
<LI><A target=content href="SaveXmlDocument.aspx">How Do I...Save XML with the XmlDocument (W3C DOM)?</A>
<br clear="left"><br>
<LI><A target=content href="StreamAnXmlDocument.aspx">How Do I...Read XML from an XmlDataDocument?</A> 
<br clear="left"><br>
<LI><A target=content href="NavigateXmlDocument.aspx">How Do I...Navigate XML with the XmlNavigator?</A>
<br clear="left"><br>
<LI><A target=content href="EditingXmlDocument.aspx">How Do I...Edit XML with the XmlNavigator?</A> </LI>
<br clear="left"><br>
</OL>

<div class="indent" style="width:660">
In this sample we are simply going to load the XmlDataDocument with XML data using the sample file 
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/LoadXmlDocument/&file=books.xml">books.xml</a>.
The code below illustrates this.
</div>

<div class="code">
<xmp>
public class XmlDocumentSample
{
    private const String m_Document = "books.xml";

    public static void Main(String[] args)
    {
        XmlDocumentSample myxmldocument = new XmlDocumentSample(args);
    }

    public XmlDocumentSample(String[] args)
    {
        try
        {
            // Load the XML from file
            Console.WriteLine ("Loading file {0} ...", m_Document);

            XmlDataDocument xmldocument = new XmlDataDocument();
            xmldocument.Load (m_Document);

            Console.WriteLine ("XmlDataDocument loaded with XML data successfully ...");
        }
        catch (Exception e)
        {
            Console.WriteLine ("Exception: {0}", e.ToString());
        }
    }
} // End class XmlDocumentSample
</xmp>
</div>

<H4>Summary</H4>
<OL>
<LI>The XmlDataDocument class provides an in memory cache for XML data.
<LI>The XmlDataDocument extends the XmlDocument (W3C DOM) class. It can be used anywhere the XmlDocument class is used.
<LI>The XmlDataDocument and the DataSet classes are closely associated, and the XmlDataDocument can be regarded as a DataSet aware XmlDocument. 
<LI>The DataDocumentNavigator class consumes XmlDataDocuments to provide implementations of common XML interfaces for integrated support
with other XML objects.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->



















































