\<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Navigate XML with the XmlNavigator?</h4>

<div class="indent" style="width:660">
This sample illustrates how to navigate XML documents with an XmlNavigator using the DocumentNavigator and DataDocumentNavigator.
The XmlNavigator class provides a cursor style model to navigate an XML in-memory tree. The XmlNavigator also provides XPath support and is the
input to XslTransforms. XPath expressions are introduced here with more detail added in the
<a href="QueryXmlDocumentXPath.aspx">How Do I...Query XML with an XPath expression?</a> topic.
</div>

<h4>Navigating XML with the XmlNavigator</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/NavigateXmlDocument/NavigateXmlDocument.src"
RunSample="/quickstart/howto/samples/Xml/NavigateXmlDocument/NavigateXmlDocument.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="NavigateXmlDocument.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
In this sample we are going to load an XmlDocument with XML data using the sample file
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/NavigateXmlDocument/&file=books.xml">books.xml</a>,
create a DocumentNavigator as a view onto the data and display the XML by moving through the document. The <b>DocumentNavigator</b> class provides an implementation
of the XmlNavigator for the XmlDocument class and the <b>DataDocumentNavigator</b> class provides an implementation of the XmlNavigator for the XmlDataDocument class.
The sample code below shows the creation of these classes.

<div class="code">
<xmp>
// Create a DocumentNavigator
XmlDocument xmldocument = new XmlDocument();
DocumentNavigator navigator = new DocumentNavigator(xmldocument);

// Create a DataDocumentNavigator
XmlDataDocument xmldatadocument = new XmlDataDocument();
DataDocumentNavigator datanavigator = new DataDocumentNavigator(xmldatadocument);
</xmp></div>

<div style="width:660">
For the XmlNavigator, the properties and methods operate in relation to the current node. The <b>MoveToNext</b> method moves to the next sibling of the 
current node. <b>The MoveToChild(i)</b> method moves to the specified child of the current node (where 0 is the first child).
The <b>MoveToParent</b> method moves to the parent node of the current node. When positioned on an attribute you must use the <b>MoveToElement</b>
method to move to the element that owns the attribute node. This is how you get back to the element after navigating through its attributes.
The <b>HasChildren</b> property indicates whether the current node has child nodes and the <b>HasAttributes</b> property indicates whether
the current node has attribute nodes. Lastly the <b>MoveToDocument</b> method sets the navigator to the document node that contains the whole tree of nodes.
These methods enable us to navigate the XML document in a recursive fashion as illustrated in the code below.</div>

<div class="code">
<xmp>
navigator.MoveToDocument(); // Initialize the Navigator to start at the root
DisplayTree(navigator); // Display all the nodes

// Walks the XmlNavigator tree recursively 
public void DisplayTree (XmlNavigator navigator)
{
    if (navigator.HasChildren)
    {
        navigator.MoveToChild(0);
        Format (navigator);ww
        DisplayTree (navigator); 
        navigator.MoveToParent();
    }
    while (navigator.MoveToNext())
    {
        Format (navigator);
        DisplayTree (navigator); 
    }
}

// Format the output
private void Format (XmlNavigator navigator)
{
    if (!navigator.HasChildren)
    {
        Console.Write("<" + navigator.Name + ">" + navigator.Value + "\r\n");
    }
    else
    {
        Console.Write("<" + navigator.Name + ">");

        if (navigator.HasAttributes)
            Console.Write("\r\nAttributes of <" + navigator.Name + ">\r\n");

        for (int i = 0; i < navigator.AttributeCount; i++)
        {
            navigator.PushPosition();
            navigator.MoveToAttribute(i);
            Console.Write("<" + navigator.Name + "> " + navigator.Value + " ");
            navigator.PopPosition();
        }
        Console.WriteLine();
    }
}
</xmp></div>

<div style="width:660">
The output from executing this code is shown below.</div>

<div class="code"><xmp>
Display contents of the file books.xml

<xml>version='1.0'
<#comment> This file represents a fragment of a book store inventory database
<bookstore>
<book>
Attributes of <book>
<genre> autobiography <publicationdate> 1981 <ISBN> 1-861003-11-0
<title>
<#text>The Autobiography of Benjamin Franklin
<author>
<first-name>
<#text>Benjamin
<last-name>
<#text>Franklin
<price>
<#text>8.99
<book>
Attributes of <book>
<genre> novel <publicationdate> 1967 <ISBN> 0-201-63361-2
<title>
<#text>The Confidence Man
<author>
<first-name>
<#text>Herman
<last-name>
<#text>Melville
<price>
<#text>11.99
<book>
Attributes of <book>
<genre> philosophy <publicationdate> 1991 <ISBN> 1-861001-57-6
<title>
<#text>The Gorgias
<author>
<name>
<#text>Plato
<price>
<#text>9.99
</xmp></div>
        
<h4>Explicit Moves</h4> 

<div style="width:660">
If the structure of the XML is known, then it can be navigated by explicitly moving through the document from node to node. The code below finds the price of the
first book in the sample file books.xml.</div> 

<div class="code"><xmp>
Console.WriteLine ("Find the price of the first book by navigating nodes ...");
navigator.MoveToDocument();
DisplayNode (true, navigator); // root node
DisplayNode (navigator.MoveToFirstChild(), navigator); // ?xml version='1.0'? node
DisplayNode (navigator.MoveToNext(), navigator); //!-- This file ... node
DisplayNode (navigator.MoveToNext(), navigator); // bookstore element
DisplayNode (navigator.MoveToFirstChild(), navigator); // book element
DisplayNode (navigator.MoveToFirstChild(), navigator); // title element
DisplayNode (navigator.MoveToNext(), navigator);// author element
DisplayNode (navigator.MoveToNext(), navigator); // price Element
DisplayNode (navigator.MoveToFirstChild(), navigator); // value of price element

private void DisplayNode(Boolean success, XmlNavigator navigator)
{
    if (success && (navigator.NodeType == XmlNodeType.Text))
        Console.Write("<" + navigator.Name + ">" + navigator.Value + "\r\n");
    else if (success)
        Console.Write("<" + navigator.Name + ">\r\n");
    else
        Console.WriteLine();
}
</xmp></div>

<div style="width:660">
The output from executing this code is shown below.</div>

<div class="code"><xmp>
Find the price of the first book by navigating nodes ...
<#document>
<xml>
<#comment>
<bookstore>
<book>
<title>
<author>
<price>
<#text>8.99
</xmp></div>

<h4>Selecting a Set of Nodes</h4> 

<div style="width:660">
One of the most powerful features of the XmlNavigator is its ability to create a set of selected nodes with an XPath expression using the <b>Select</b>
method. XPath is a language that provides filtering and addressing of data in an XML document and is a more intuative approach to retrieving a set of nodes.
See <a href="QueryXmlDocumentXPath.aspx">How Do I...Query XML with an XPath expression?</a>
The XmlNavigator can apply a query and generate a set of nodes that correspond to the selected query set and these can be navigated independently. The methods
<b>MoveToFirstSelected</b> and <b>MoveToNextSelected</b> allow navigation over this selected tree. Note that from any of the selected nodes it is possible
to navigate to the next sibling of the selected node using <b>MoveNext</b> method or any of the other navigation methods. I.e. You can move between the 
selected nodes and the whole tree of nodes depending upon the navigation methods used. The code example below selects and displays all the book titles from 
the sample file books.xml.</div>

<div class="code">
<xmp>
navigator.MoveToDocument();
navigator.Select("descendant::book/title");

while(navigator.MoveToNextSelected())
{
    Console.Write("<" + navigator.Name + ">" + navigator.InnerText + "\r\n");
};
</xmp></div>

<div style="width:660">
The <b>InnerText</b> property on a node, either gets the concatenation for the text values of all it's children as a string, or replaces all the child nodes
with a single text node that has a value of the supplied string. It is used here since the selected elements only have a single child text node for the 
title of the book. The output from executing this code is shown.</div>

<div class="code">
<xmp>
Select the book titles ...
<title>The Autobiography of Benjamin Franklin
<title>The Confidence Man
<title>The Gorgias
</xmp></div>

<H4>Summary</H4>
<OL>
<LI>The XmlNavigator provides a cursor style model to navigate an XML in-memory tree. 
<LI>The DocumentNavigator and DataDocumentNavigator classes are implementations of the XmlNavigator.
<LI>The MoveTo methods allow you to perform random access operations on the XML document, and to directly walk the XML nodes.
<LI>The Select method, using XPath expressions, is a powerful method of creating a set of selected nodes.
<LI>The MoveToNextSelected method moves across the tree of selected nodes.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->

</xml>


































