<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" --><h4>How Do I...Query XML with an XPath expression?</h4>

<div class="indent" style="width:660">
This sample illustrates how to query an XmlDataDocument or an XmlDocument with an XPath expression using the XmlNavigator class.
This sample follows on from the <a href="NavigateXmlDocument.aspx">How Do I...Navigate XML with the XmlNavigator?</a> topic.
The XPath queries conform to the <A href="http://www.w3.org/TR/xpath">W3C XML Path Language (XPath)</A> specification.
</div>

<h4>Querying XML with an XPath expression</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/QueryXmlDocumentXPath/QueryXmlDocumentXPath.src"
RunSample="/quickstart/howto/samples/Xml/QueryXmlDocumentXPath/QueryXmlDocumentXPath.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="QueryXmlDocumentXPath.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
XPath is the W3C's general query language specification for addressing parts of an XML document. In this sample we are going to load an XmlDocument with the sample
file <a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/QueryXmlDocumentXPath/&file=books.xml">books.xml</a>,
perform an XPath query on XML and navigate over the selected nodes using an XmlNavigator. The sample code below illustrates the <b>Select</b> method that
applies an XPath expression.<div>

<div class="code">
<xmp>
XmlDocument xmldocument = new XmlDocument();
xmldocument.Load (m_Document);
DocumentNavigator navigator = new DocumentNavigator(xmldocument);

// Get all the book prices
XPathQuery(navigator, "descendant::book/price");
// Get the ISBN of the last book
XPathQuery(navigator, "//book[last()]/@ISBN/text()");

private void XPathQuery(XmlNavigator navigator, String xpathexpr )
{
    try
    {
        // Save context node position
        navigator.PushPosition();
        Console.WriteLine("XPath query: " + xpathexpr);

        navigator.Select (xpathexpr);

        FormatXml(navigator);

        // Restore context node position
        navigator.PopPosition();        
    }
    catch (Exception e)
    {
        Console.WriteLine ("Exception: {0}", e.ToString());
    }
}
</xmp></div>
<div class="indent" style="width:660">
The first query selects all the 'price' nodes for all the books starting from the root node. The second query gets the 'ISBN' attribute text
for the last book element node starting from the root node.</div> 

<br clear="left"><div class="indent" style="width:660">
The <b>PushPosition</b> method remembers the current position of the XmlNavigator on a stack. The selection state however, is not remembered. You can later
return to this position by calling the <b>PopPosition</b> method. This enables you to move through the document using one select query and then return to a
known position, to perform another query. In other words, PushPosition remembers the node you were on when called, so that when you call PopPosition, you can
be taken back to that node. The output from these queries is shown.</div> 

<div class="code">
<xmp>
XPath query: descendant::book/price
Element<price>8.99
Element<price>11.99
Element<price>9.99

XPath query: //book[last()]/@ISBN/text()
Text<#text>1-861001-57-6
</xmp>
</div>

<H4>Summary</H4>
<OL>
<LI>XPath is the W3C's general query language specification for addressing parts of an XML document and is a powerful XML query language.
<LI>The Select method applies the XPath query to the XML document.
<LI>The MoveToNextSelected method moves between the nodes returned from the XPath query.
<LI>The PushPosition method remembers the current position of the XmlNavigator on a stack. The selection state however, is not remembered. You can later
return to this position by calling the PopPosition method.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->






















