<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Read XML from a file?</h4>

<div class="indent" style="width:660">
This sample illustrates how to read XML from a file using the XmlTextReader class.
This class provides direct parsing and tokenizing of XML and implements the <A href="http://www.w3.org/TR/1998/REC-xml-19980210">W3C 
Extensible Markup Language (XML) 1.0</A> and the <A><A href="http://www.w3.org/TR/REC-xml-names/">Namespaces in XML</A></A> specifications.
This reader provides fast, tokenized, stream access to XML rather than using an object model such as the XML DOM.
See <A target=content href="DOMInterfaceXmlDocument.aspx">How Do I...Create and use the XmlDocument (W3C DOM)?</a></div>

<br clear="left"><div class="indent" style="width:660">
The XmlReader class is the API which provides the XML parsing. The XmlTextReader is an implementation of this API to handle byte streams.
</div>

<h4>Reading XML from a file</h4> 
<Acme:SourceRef 
ViewSource="/quickstart/howto/samples/Xml/ReadXmlFile/ReadXmlFile.src"
RunSample="/quickstart/howto/samples/Xml/ReadXmlFile/ReadXmlFile.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="ReadXmlFile.aspx"
runat="server" />

<br clear="left"><br><div class="indent" style="width:660">
Typically the XmlTextReader is used if you need to access the XML as 'raw' data without the overhead of a DOM and therefore provides a faster 
mechanism for reading XML.  For example an XML document could have a header section used for routing the document for processing elsewhere.
The XmlTextReader has different constructors to specify the location of the XML data. In this sample we are going to load XML from the
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/ReadXmlFile/&file=books.xml">books.xml</a> file.
The sample code shown below constructs an XmlTextReader.

<div class="code"><xmp>
XmlTextReader reader = new XmlTextReader ("books.xml");
</xmp></div>

<div style="width:660">
Once loaded, the XmlTextReader moves across the XML data by performing sequential reads to get the next record using the <b>Read</b> method. It returns false
if there are no more records.
<div class="code">
<xmp>
while (reader.Read())
{
    // Do some work here on the data
}
</xmp></div>

<div style="width:660">
To processes the XML data, each record has a node type which can be determined from the <b>NodeType</b> property. The <b>Name</b> and <b>Value</b> properties return
the node name (e.g. the element and attribute names) and the node value (i.e. the node text) of the current node (or record). The code sample below uses these
properties to display the details about the node for Element and DocumentType types. The node type is determined by the NodeType enumeration shown
in the table.

<div class="code"><xmp>
while (reader.Read())
{
    switch (reader.NodeType)
    {
    case XmlNodeType.Element: // The node is an Element
        Console.WriteLine(NodeType + "<" + reader.Name + ">" + reader.Value);
        break;
    case XmlNodeType.DocumentType: // The node is a DocumentType
        Console.WriteLine(NodeType + "<" + reader.Name + ">" + reader.Value);
        break;
    }
}
</xmp></div>

<div>Table of NodeTypes which are equivalent to the W3C DOM node types with some extended types required for forward only reading.</div><br clear="left"><br>

<DIV class=indent>
<TABLE class=table 
style="border-style: solid"
width="418">
<TBODY>
<TR>
<TH width="100">NodeType Enum</TH>
<TH width="308">Description</TH>
<TH width="10">Value</TH>
</TR>
  <tr>
    <td height="17"><font size="1">None</font></td>
    <td height="17"><font size="1"></font></td>
    <td height="17"><font size="1">0</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Element</font></td>
    <td width="308" height="19"><font size="1">&lt;name&gt;</font></td>
    <td width="10" height="17"><font size="1">1</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Attribute</font></td>
    <td width="308" height="19"><font size="1">id='123'</font></td>
    <td width="10" height="17"><font size="1">2</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Text</font></td>
    <td width="308" height="19"><font size="1">'123'</font></td>
    <td width="10" height="17"><font size="1">3</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">CDATA</font></td>
    <td width="308" height="19"><font size="1">&lt;![CDATA[....]]&gt;</font></td>
    <td width="10" height="17"><font size="1">4</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">EntityReference</font></td>
    <td width="308" height="19"><font size="1">&amp;foo;</font></td>
    <td width="10" height="17"><font size="1">5</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Entity</font></td>
    <td width="1000" height="19"><font size="1">&lt;!ENTITY ...&gt;</font></td>
    <td width="10" height="17"><font size="1">6</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">ProcessingInstruction</font></td>
    <td width="1000" height="19"><font size="1">&lt;?pi test?&gt;</font></td>
    <td width="10" height="17"><font size="1">7</font></td>
  </tr>
  <tr>
    <td height="17"><font size="1">Comment</font></td>
    <td height="17"><font size="1">&lt;!-- comment --&gt;</font></td>
    <td height="17"><font size="1">8</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Document</font></td>
    <td width="308" height="19"><font size="1"></font></td>
    <td width="10" height="17"><font size="1">9</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">DocumentType</font></td>
    <td width="308" height="19"><font size="1">&lt;!DOCTYPE ...&gt;</font></td>
    <td width="10" height="17"><font size="1">10</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">DocumentFragment</font></td>
    <td width="308" height="19"><font size="1"></font></td>
    <td width="10" height="17"><font size="1">11</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Notation</font></td>
    <td width="308" height="19"><font size="1">&lt;!NOTATION ...&gt;</font></td>
    <td width="10" height="17"><font size="1">12</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">Whitespace</font></td>
    <td width="308" height="19"><font size="1">Whitespace between markup.</font></td>
    <td width="10" height="17"><font size="1">13</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">SignificantWhitespace</font></td>
    <td width="1000" height="19"><font size="1">Whitespace between markup in a mixed content model.</font></td>
    <td width="10" height="17"><font size="1">14</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">EndTag</font></td>
    <td width="1000" height="19"><font size="1">&lt;/foo&gt;</font></td>
    <td width="10" height="17"><font size="1">15</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">EndEntity</font></td>
    <td width="1000" height="19"><font size="1">Returned when the reader has gotten to the end of the entity replacement as a result of a call to ExpandEntity().</font></td>
    <td width="10" height="17"><font size="1">16</font></td>
  </tr>
  <tr>
    <td width="100" height="19"><font size="1">CharacterEntity</font></td>
    <td width="1000" height="19"><font size="1">Returned when the reader has been told to report character entities (e.g. &amp;#65;). See the 
    EntityHandling property.</font></td>
    <td width="10" height="17"><font size="1">17</font></td>
  </tr>
  </TBODY></TABLE></DIV>
<br clear="left"><br>

<div style="width:660">
The <b>Depth</b> property reports the depth of the current node and can be useful for formatting. Nodes at the root level are at depth 0. Combining this with
the Name and Value properties we can create a sample which processes an XML file and formats the output depending on the node type and the depth,
gathering statistics as it reads. The Format method, shown below, implements some basic formatting code to the console. The full code is at 
<a href="/quickstart/util/srcview.aspx?path=/quickstart/howto/samples/Xml/ReadXmlFile/ReadXmlFile.src">View Source</a>.</div>

<div class="code"><xmp>
private static void Format(XmlReader reader, String NodeType)
{
    // Format the output
    Console.Write(reader.Depth + " ");
    Console.Write(reader.AttributeCount + " ");
    for (int i=0; i < reader.Depth; i++)
    { 
        Console.Write('\t');
    }

    Console.Write(reader.Prefix + NodeType + "<" + reader.Name + ">" + reader.Value);

    // Display the attributes values for the current node
    if (reader.HasAttributes)
    {
        Console.Write(" Attributes:");

        for (int j=0; j < reader.AttributeCount; j++)
        {
            Console.Write(" [{0}] " + reader[j], j);
        }
    }
    Console.WriteLine();
}
</xmp></div>
<div style="width:660">
The <b>Prefix</b> property returns the namespace prefix associated with the node. Element node types can have a list of attribute nodes associated with them.
Here we test whether the node has any attributes with the <b>HasAttributes</b> property and then use the node index operators to retrieve each attribute value.
This is analogous to a collection of attributes for the node.
The <b>AttributeCount</b> property returns the number of attributes for the current node. This approach is used if all you are interested in are the attribute
values and are not concerned with other properties of the attribute nodes (e.g. The name of the attribute).
In the <A target=content href="ReadXmlStream.aspx">How Do I...Read XML from a stream?</a> topic we show an alternative approach to accessing the attributes
by moving to each attribute node in order to read both its name and value.</div>

<br clear="left"><div style="width:660">
The output from running this sample with the 
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/ReadXmlFile/&file=books.xml">books.xml</a> file
is shown below. The first column is the Depth property and the second column is the AttributeCount property.</div>

<div class="code"><xmp>
0 0 ProcessingInstruction<xml>version='1.0'
0 0 Comment<> This file represents a fragment of a book store inventory database
0 0 Element<bookstore>
1 3     Element<book> Attributes: [0] autobiography [1] 1981 [2] 1-861003-11-0
2 0             Element<title>
3 0                     Text<>The Autobiography of Benjamin Franklin
2 0             Element<author>
3 0                     Element<first-name>
4 0                             Text<>Benjamin
3 0                     Element<last-name>
4 0                             Text<>Franklin
2 0             Element<price>
3 0                     Text<>8.99
1 3     Element<book> Attributes: [0] novel [1] 1967 [2] 0-201-63361-2
2 0             Element<title>
3 0                     Text<>The Confidence Man
2 0             Element<author>
3 0                     Element<first-name>
4 0                             Text<>Herman
3 0                     Element<last-name>
4 0                             Text<>Melville
2 0             Element<price>
3 0                     Text<>11.99
1 3     Element<book> Attributes: [0] philosophy [1] 1991 [2] 1-861001-57-6
2 0             Element<title>
3 0                     Text<>The Gorgias
2 0             Element<author>
3 0                     Element<name>
4 0                             Text<>Plato
2 0             Element<price>
3 0                     Text<>9.99

Statistics for books.xml file

ProcessingInstruction: 1
DocumentType: 0
Comment: 1
Element: 18
Attribute: 9
Text: 11
Whitespace: 27
</xmp></div>

<H4>Summary</H4>
<OL>
<LI>The XmlTextReader provides fast, non-cached, forward only read access to XML data.
<LI>The XmlTextReader implements the <A href="http://www.w3.org/TR/1998/REC-xml-19980210">W3C Extensible Markup 
Language (XML) 1.0</A></A> specification and the <A><A href="http://www.w3.org/TR/REC-xml-names/">Namespaces in XML</A></A> specification. 
<LI>The XmlTextReader provides constructors to read XML from a file, a stream or a TextReader.
<LI>The Read method moves the reader sequentially through the records (or nodes).
<LI>For element nodes, the value of an attribute can be obtained by using the index operators.
<LI>Attributes are represented as a node list off the current node and can be discovered through the HasAttributes property.
<LI>The Depth property reports the depth of the current node and can be useful for formatting. Nodes at the root level are at depth 0.
<LI>The Name and Value properties provide details about the current node.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->



































































































