<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Read XML data from a stream?</h4>

<div class="indent" style="width:660">
This sample illustrates how to read XML from a stream using the XmlTextReader class. The stream could have come from a variety of sources such as a byte stream
from a server, a file or from a TextReader. <br clear="left"><br>
This sample follows on from the <a href="/quickstart/howto/doc/Xml/ReadXmlFile.aspx">How do I...Read XML from a file?</a> topic.
</div>

<h4>Reading XML from a stream</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/ReadXmlStream/ReadXmlStream.src"
RunSample="/quickstart/howto/samples/Xml/ReadXmlStream/ReadXmlStream.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="ReadXmlStream.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
The XmlTextReader has different constructors to specify the location of the XML data. Here we are going to load the XmlTextReader from a stream.
A stream is an abstract representation of an input or output device that is the source of, or destination for, data (in this case XML data). You can write to 
a stream and read from a stream, which is best visualised as a flow of bytes. Streams are used to provide independence from the device and hence require
no program changes if, for instance, the source of a stream changes.

<br clear="left"><br>
<div style="width:660">
The sample code shown below creates a StringReader class which is then used to build up an XML string. Since this is purely a byte stream which is held in memory,
we can get the XmlTextReader to parse this stream as XML. This in memory stream has no particular specified encoding. An XmlTextReader is then created which parses
the stream and displays the resultant XML.

<div class="code"><xmp>
StringReader stream;
XmlTextReader reader = null;

stream = new StringReader("<?xml version='1.0'?>" +
                          "<!-- This file represents a fragment of a book store inventory database -->" +
                          "<bookstore>" +
                          " <book genre=\"autobiography\" publicationdate=\"1981\" ISBN=\"1-861003-11-0\">" +
                          "   <title>The Autobiography of Benjamin Franklin</title>" +
                          "   <author>" +
                          "       <first-name>Benjamin</first-name>" +
                          "       <last-name>Franklin</last-name>" +
                          "   </author>" +
                          "   <price>8.99</price>" +
                          " </book>" +
                          " <book genre=\"novel\" publicationdate=\"1967\" ISBN=\"0-201-63361-2\">" +
                          "   <title>The Confidence Man</title>" +
                          "   <author>" +
                          "       <first-name>Herman</first-name>" +
                          "       <last-name>Melville</last-name>" +
                          "   </author>" +
                          "   <price>11.99</price>" +
                          " </book>" +
                          "  <book genre=\"philosophy\" publicationdate=\"1991\" ISBN=\"1-861001-57-6\">" +
                          "   <title>The Gorgias</title>" +
                          "   <author>" +
                          "       <name>Plato</name>" +
                          "   </author>" +
                          "   <price>9.99</price>" +
                          " </book>" +
                          "</bookstore>");

// Load the XmlTextReader from the stream
reader = new XmlTextReader (stream);

Console.WriteLine ("Processing ...");
Console.WriteLine ();
FormatXml(reader);

// Finished with XmlTextReader
if (reader != null)
    reader.Close();
</xmp></div>

<div style="width:660">
If a Stream is supplied as input, the XmlTextReader properly decodes this by wrapping the stream in a StreamReader and calling the SwitchEncoding property according 
to the XML encoding specified. Also the XmlResolver is used to resolve external resources needed to correctly parse the input - like DTD's, schemas, entities
and &lt;xml:include&gt;s. Another method of representing a stream is by using a StreamReader class that implements a reader which reads from a character stream.
The code sample below illustrates loading a file named books.xml and then using the XmlTextReader to parse the resulting file.</div>

<div class="code"><xmp>
StreamReader streamreader = new StreamReader ("books.xml");
Console.WriteLine ("File books.xml read sucessfully ...");

// Load the XmlTextReader from the StreamReader
XmlTextReader xmlreader = new XmlTextReader (streamreader);
</xmp></div>

<div style="width:660">
This FormalXml method in this sample shows how to move to the attribute nodes using the <b>MoveToNextAttribute</b> method when the current node is an element
node. This allows you not only to access the node's name and value properties but, because you are at a node context you can also get properties like the current
namespace for the attribute. The code sample below also shows the Format method that this time displays the name and value for the current node.</div>

<div class="code"><xmp>
private static void FormatXml (XmlReader reader)
{
    int PI_count=0, Doc_count=0, comment_count=0, element_count=0, attribute_count=0, text_count=0, whitespace_count=0;

    while (reader.Read())
    {
        switch (reader.NodeType)
        {
        case XmlNodeType.Element:
            Format (reader, "Element");
            while(reader.MoveToNextAttribute())
            {
                Format (reader, "Attribute");
            }
            element_count++;
            if (reader.HasAttributes)
                attribute_count += reader.AttributeCount;
            break;
         }
    }
}

// Format the output
private static void Format(XmlReader reader, String NodeType)
{
    // Format the output
    Console.Write(reader.Depth + " ");
    Console.Write(reader.AttributeCount + " ");
    for (int i=0; i < reader.Depth; i++)
    { 
        Console.Write('\t');
    }

    Console.Write(NodeType + "<" + reader.Name + ">" + reader.Value);
    Console.WriteLine();
}
</xmp></div>

<H4>Summary</H4>
<OL>
<LI>A stream is an abstract representation of an input or output device that is the source of, or destination for, data.
<LI>The XmlTextReader provides constructors to read XML from a file, a stream or a TextReader.
<LI>The XmlTextReader properly decodes the stream by wrapping the stream in a StreamReader and setting the specified XML encoding.
<LI>Attribute nodes can be accessed with the MoveToNextAttribute method that allows you to determine the properties of the attribute node.
</OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
</xml>





































