<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Save DataSet mappings to an XSD schema file?</h4>

<div class="indent" style="width:660">
This sample illustrates how to save the internal DataSet mappings to an XSD schema file. In this sample the mappings are built by using the relational methods
on the DataSet to create tables and columns and an XSD schema representation for these is then written out to a file.</div>

<h4>Saving DataSet mappings to an XSD schema file</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/SaveDataSetMapXSDSchema/SaveDataSetMapXSDSchema.src"
RunSample="/quickstart/howto/samples/Xml/SaveDataSetMapXSDSchema/SaveDataSetMapXSDSchema.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="SaveDataSetMapXSDSchema.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
The DataSet and XmlDataDocument classes both represent an in-memory data cache. The XmlDataDocument provides XML navigational and editing methods whilst the DataSet
provides relationally orientated navigational and editing methods. Here we are going to get the DataSet property from the XmlDataDocument, use it to build a set of
tables and columns and then populate them. We can then write out the internally generated schema. The code illustrated below builds up two tables, one of people
and the other of pets. An ID is used as a primary key onto the each of the tables and a relationship table is built between the people and their pets.
</div>

<br clear="left"><div class="indent" style="width:660">
First we create an instance of an XmlDataDocument, and pass it's DataSet to the LoadDataSet() method.
</div>

<div class="code"><xmp>
XmlDataDocument datadoc = new XmlDataDocument();
LoadDataSet(datadoc.DataSet);
</xmp></div>

<div class="indent" style="width:660">
The LoadDataSet() method loads the dataset with data relationally.
</div>

<div class="code"><xmp>
// Load a DataSet with relational data
private void LoadDataSet(DataSet dataset)
{
    try
    {
        Console.WriteLine("Loading the DataSet ...");

        // Set DataSet name
        dataset.DataSetName = "PersonPet";

        // Create tables for people and pets
        DataTable people = new DataTable("Person");
        DataTable pets = new DataTable("Pet");

        // Set up the columns in the Tables
        DataColumn personname = new DataColumn ("Name", typeof(String));
        DataColumn personAge = new DataColumn ("Age", typeof(Int32));

        DataColumn petname = new DataColumn ("Name", typeof(String));
        DataColumn pettype = new DataColumn ("Type", typeof(String));

        // Add columns to person table
        DataColumn id = people.Columns.Add("ID", typeof(Int32));
        id.AutoIncrement = true;
        people.PrimaryKey = new DataColumn[] {id};
        people.Columns.Add (personname);
        people.Columns.Add (personAge);

        // Add columns to pet table
        id = pets.Columns.Add("ID", typeof(Int32));
        id.AutoIncrement = true;
        pets.PrimaryKey = new DataColumn[] {id};
        id.AutoIncrement = true;
        DataColumn ownerid = pets.Columns.Add("OwnerID", typeof(Int32));
        DataColumn[] foreignkey = new DataColumn[] {ownerid};
        pets.Columns.Add (petname);
        pets.Columns.Add (pettype);

        // Add tables to the DataSet
        dataset.Tables.Add (people);
        dataset.Tables.Add (pets);

        // Add people
        DataRow mark = people.NewRow();
        mark[personname] = "Mark";
        mark[personAge] = 18;
        people.Rows.Add(mark);

        DataRow william = people.NewRow();
        william[personname] = "William";
        william[personAge] = 12;
        people.Rows.Add(william);

        DataRow james = people.NewRow();
        james[personname] = "James";
        james[personAge] = 7;
        people.Rows.Add(james);

        DataRow levi = people.NewRow();
        levi[personname] = "Levi";
        levi[personAge] = 4;
        people.Rows.Add(levi);

        // Add relationships
        Console.WriteLine("Creating relationships between people and pets ...");
        DataRelation personpetrel = new DataRelation ("PersonPet",people.PrimaryKey, foreignkey);
        dataset.Relations.Add (personpetrel);

        // Add pets
        DataRow row = pets.NewRow();
        row["OwnerID"] = mark["ID"];
        row[petname] = "Frank";
        row[pettype] = "cat";
        pets.Rows.Add(row);

        row = pets.NewRow();
        row["OwnerID"] = william["ID"];
        row[petname] = "Rex";
        row[pettype] = "dog";
        pets.Rows.Add(row);

        row = pets.NewRow();
        row["OwnerID"] = james["ID"];
        row[petname] = "Cottontail";
        row[pettype] = "rabbit";
        pets.Rows.Add(row);

        row = pets.NewRow();
        row["OwnerID"] = levi["ID"];
        row[petname] = "Sid";
        row[pettype] = "snake";
        pets.Rows.Add(row);

        row = pets.NewRow();
        row["OwnerID"] = levi["ID"];
        row[petname] = "Tickles";
        row[pettype] = "spider";
        pets.Rows.Add(row);

        row = pets.NewRow();
        row["OwnerID"] = william["ID"];
        row[petname] = "Tweetie";
        row[pettype] = "canary";
        pets.Rows.Add(row);
                
        // commit changes
        dataset.AcceptChanges();
    }

    catch (Exception e)
    {
        Console.WriteLine("Exception: ", e.ToString());
    }
}    
</xmp></div>

<div class="indent" style="width:660">
The DataSet <b>AcceptChanges</b> method commits all the changes that have been made to this DataSet since it was loaded or the last time AcceptChanges was called.
All new and modified rows become unchanged, and deleted rows get removed. For more detail on the other DataSet relational methods see
<A target=content href="/quickstart/howto/doc/adoplus/adoplusoverview.aspx">How do I...Get an Overview of ADO+?</A></div>

<br clear="left"><div class="indent" style="width:660">
To save the schema to a file the DataSet <b>WriteXmlSchema</b> method is called passing a StreamWriter class that represents the destination file.
</div>

<div class="code"><xmp>
StreamWriter writer = new StreamWriter(m_Schema);
datadoc.DataSet.WriteXmlSchema(writer);
writer.Close();
</xmp></div>

<div class="indent" style="width:660">
The output from the DisplayTables() method for the tables built in the DataSet is shown below and the schema is written to the
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/SaveDataSetMapXSDSchema/&file=PersonPet.xsd">PersonPet.xsd</a> file.
</div>

<div class="code"><xmp>
Loading the DataSet ...
Creating relationships between people and pets ...

DataSet:
PersonPet contains ...
No of Tables: 2  Table content ...

TableName = Person
---------
Columns ...

ID                    Name                  Age
Number of rows = 4
Rows ...

0                     Mark                  18
1                     William               12
2                     James                 7
3                     Levi                  4

TableName = Pet
---------
Columns ...

ID                    OwnerID               Name                  Type
Number of rows = 6
Rows ...

0                     0                     Frank                 cat
1                     1                     Rex                   dog
2                     2                     Cottontail            rabbit
3                     3                     Sid                   snake
4                     3                     Tickles               spider
5                     1                     Tweetie               canary

PersonPet

Name = Mark owns
        Pet = Frank the cat

Name = William owns
        Pet = Rex the dog
        Pet = Tweetie the canary

Name = James owns
        Pet = Cottontail the rabbit

Name = Levi owns
        Pet = Sid the snake
        Pet = Tickles the spider

Writing the schema to PersonPet.xsd ...
</xmp></div>

<H4>Summary</H4>
<OL>
<LI>The WriteXmlSchema method saves the mappings for the internal structure of the relation data in the DataSet as an XSD Schema.
<LI>The XmlDataDocument has a DataSet property that enables you to view and manage structured data relationally within an XML document.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
