<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Save XML with the XmlDocument (W3C DOM)?</h4>

<div class="indent" style="width:660">
This sample illustrates how to update and save XML with the XmlDocument (W3C DOM) class.<br clear="left"><br>
This sample follows on from the <a href="DOMInterfaceXmlDocument.aspx">How do I...Load and use the XmlDocument (W3C DOM)?</a> topic.
</div>

<h4>Saving XML with the XmlDocument</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/SaveXmlDocument/SaveXmlDocument.src"
RunSample="/quickstart/howto/samples/Xml/SaveXmlDocument/SaveXmlDocument.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="SaveXmlDocument.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
The XmlDocument class provides the ability to save XML data to files, streams and XmlWriters via the <b>Save</b> method. In this sample we are going to navigate over
the sample file <a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/SaveXmlDocument/&file=books.xml">books.xml</a>,
increase all the books prices by 2% and save the XML to a new file called
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/SaveXmlDocument/&file=updatebooks.xml">updatebooks.xml</a>
as shown in the code below. </div>

<div class="code">
<xmp>
// Create XmlDocument and load the XML from file
XmlDocument xmldocument = new XmlDocument();
xmldocument.Load (new XmlTextReader (m_Document));
Console.WriteLine ("XmlDocument loaded with XML data successfully ...");

IncreasePrice(xmldocument.DocumentElement);

// Write out data as XML
xmldocument.Save(m_UpdatedDocument);
Console.WriteLine();
Console.WriteLine("Updated prices saved in file {0}", m_UpdatedDocument);
</xmp></div>

<div class="indent" style="width:660">
The IncreasePrice method recursively iterates over the XML document using the <b>FirstChild</b> method to move down the tree. This method returns null if
there are no child nodes. The <b>NextSibling</b> property moves to the node immediately next to the current node returning null if there is no node to move to.
Every time a node named 'price' is encountered, the price is updated. The code below illustrates this.
</div>

<div class="code">
<xmp>
public void IncreasePrice(XmlNode node)
{       
    if (node.Name == "price")
    {
        node = node.FirstChild;
        Decimal price = Decimal.Parse(node.Value);
        // Increase all the book prices by 2%
        String newprice = Decimal.Format(price*(new Decimal(1.02)),"#.00");
        Console.WriteLine("Old Price = " + node.Value + "\tNew price = " + newprice);
        node.Value = newprice;
    }

    node = node.FirstChild;
    while (node != null)
    {
        IncreasePrice(node);
        node = node.NextSibling;
    }
}
</xmp>
</div>

<div class="indent" style="width:660">
Another useful property is the <b>HasChildNodes</b> property, that determines whether the current node has any children or not.
The output from running this sample is shown below.</div>

<div class="code"><xmp>
XmlDocument loaded with XML data successfully ...

Old Price = 8.99        New price = 9.17
Old Price = 11.99       New price = 12.23
Old Price = 9.99        New price = 10.19

Updated prices saved in file updatebooks.xml
</xmp></div>

<H4>Summary</H4>
<OL>
<LI>The XmlDocument class can save XML to a file, stream or an XmlWriter.
<LI>The XmlNode class enables you to navigate and amend the nodes within an XML document.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
