<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Read XML from an XmlDataDocument?</h4>

<div class="indent" style="width:660">
This sample illustrates how to read XML from an XmlDataDocument by creating an XmlReader class
that reads the XML data stored in the XmlDataDocument.
This sample follows on from the <a href="LoadXmlDocument.aspx">How do I...Load XML data into an XmlDataDocument?</a> topic.</div>

<h4>Reading XML from an XmlDataDocument</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/StreamXmlDocument/StreamXmlDocument.src"
RunSample="/quickstart/howto/samples/Xml/StreamXmlDocument/StreamXmlDocument.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="StreamXmlDataDocument.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
As mentioned in the previous topic, the XmlDataDocument class is an in-memory cache for XML data. The DataDocumentReader provides an implementation of an XmlReader
and consumes an XmlDataDocument to provide a mechanism for reading the data directly rather than using the W3C DOM methods. However the DataDocumentReader is not yet
implemented so an alternative approach is demonstrated here to obtain an XmlReader. In this sample we are going to load the XmlDataDocument with XML data using
the sample file <a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/StreamXmlDocument/&file=books.xml">books.xml</a>
and display the loaded XML with an XmlReader. The XmlReader allows you to perform a sequential read on the XML data.
See <a href="ReadXmlFile.aspx">How do I...Read XML from a file?</a> topic. The sample code below illustrates this.</div>

<div class="code">
<xmp>
private const String m_Document = "books.xml";
private const String m_Stylesheet = "identity.xsl";

XmlReader docreader = null;

try
{
    // Load the XML from file
    Console.WriteLine ("Loading file {0} ...", m_Document);

    XmlDataDocument xmldocument = new XmlDataDocument();
    xmldocument.Load (m_Document);

    Console.WriteLine ("XmlDataDocument loaded with XML data successfully ...\r\n");

    // The use of an XslTransform is a work-around for getting an XmlReader from XmlDataDocument
    // until DataDocumentReader is implemented.
    // DataDocumentReader docreader = DataDocumentReader(xmldocument);

    Console.WriteLine ("Reading XML ...\r\n");

    XslTransform   xsltransform = new XslTransform();
    xsltransform.Load(m_Stylesheet);
    docreader = xsltransform.Transform(new DataDocumentNavigator(xmldocument), null);
    FormatXml (docreader);
}

catch (Exception e)
{
    Console.WriteLine ("Exception: {0}", e.ToString());
}

finally
{
    // Close the reader
    if (docreader != null)
        docreader.Close();
}
</xmp>
</div>

<div class="indent" style="width:660">
Here we create an XslTransform object and load it with a style sheet,
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/StreamXmlDocument/&file=identity.xsl">identity.xsl</a>,
which replicates all the XML from the root node. Then by creating a DataDocumentNavigator onto the XmlDataDocument we can perform a transform on all the nodes
and be returned a reader onto the transformed data. This reader can now be used to read the XML data in the XmlDataDocument.
The DataDocumentNavigator class is covered in <A target=content href="NavigateXmlDocument.aspx">How Do I...Navigate XML with the XmlNavigator?</A> and 
the XslTransform class is covered in <A target=content href="TransformXml.aspx">How Do I...Apply an XSL transformation to XML?</A></div>

<H4>Summary</H4>
<OL>
<LI>An XmlReader can be obtained for the XML data in the XmlDataDocument via an XslTransform.
<LI>The DataDocumentReader class will provide an implementation of the XmlReader class to perform sequential reads on the XML data in an XmlDataDocument.
<LI>The DataDocumentReader provides and alternative, faster mechanism for reading the XML from the in-memory cache, rather than using the DOM classes and methods.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->






