<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Apply an XSL Transformation to XML?</h4>

<div class="indent" style="width:660">
This sample illustrates how to apply an XSL transformation to an XML document using the XslTransform class.
The eXtensible Stylesheet Language is an XML based language designed to transform one XML document into another XML document or an XML document into any other
structured document. This class implements the <A><A href="http://www.w3.org/TR/xslt">W3C XSL 
Transformations (XSLT) </A></A>specification. Also see the <A href="http://www.w3.org/TR/xsl/">W3C Extensible Stylesheet Language (XSL)</A></A>
</div>

<h4>Applying an XSL Transformation to XML</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/TransformXML/TransformXML.src"
RunSample="/quickstart/howto/samples/Xml/TransformXML/TransformXML.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="TransformXML.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
XSL is a transformation language based upon XPath
(see <a href="/quickstart/howto/doc/Xml/QueryXmlDocumentXPath.aspx">How Do I...Query XML with an XPath expression?</a>) to perform queries
on an XML document in order to specify a particular part of the document. XSL is required as, although XML documents are structured, there are many times when people
require that structure to be in a different format. To do this we use an XSLT processor and an XSL style sheet that tells us how to do the transformation. The style sheet
is made up of templates that specify how each node of the source XML document should appear in the destination XML document. Thus the XSLT processor holds three trees,
one for the source XML, one for the destination structure and a third for the style sheet. Two common applications of XSL are transforming one XML document into 
another XML document or the transformation of XML in HTML for display purposes.</div>

<br clear="left">
<div class="indent" style="width:660">
The diagram below illustrates the transformation process, taking in an XML document, applying the stylesheets on the XML nodes and generating another 
document of the desired format.</div>

<div class="indent" style="width:660">
<H4><img border="0" class="indent" src="XSL.gif" width="569" height="359"></H4>
</div>

<div class="indent" style="width:660">
In this sample we are going to transform one XML document into another XML document. First we load the XML document we are going to perform the transform on
into an XmlDocument. Then we create a DocumentNavigator to get an XmlNavigator that is capable of navigating the XML document.
The XslTransform object is then loaded with the style sheet using the <b>Load</b> method. The transformation is initiated by calling the <b>Transform</b>
method passing in the XmlNavigator and returning an XmlReader, that is the result of the transformation. This is shown in the code below.</div>

<div class="code">
<xmp>
public void ReadTransform()
{
    try
    {
        XmlDocument xmldocument = new XmlDocument();
        xmldocument.Load (m_Document);
        DocumentNavigator navigator = new DocumentNavigator(xmldocument);

        XslTransform   xsltransform = new XslTransform();
        xsltransform.Load(m_Stylesheet);
        XmlReader reader = xsltransform.Transform(navigator, null);

        FormatXml(reader);
    }
    catch (Exception e)
    {
        Console.WriteLine ("Exception: {0}", e.ToString());
    }
}
</xmp></div>

<div class="indent" style="width:660">
The XslTransform loads the XSLT stylesheet using an XmlReader and transforms the input data using an XmlNavigator.
</div>

<br clear="left"><div class="indent" style="width:660">
In this sample the style sheet 
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/TransformXML/&file=books.xsl">books.xsl</a>
consists of templates that get all the books in the bookstore element and then for each book element get the values for the ISBN attribute and the price element.
There is also a template that tells the XSL processor to start processing from the root ("/") node. The output from applying this style sheet is shown.
</div>

<div class="code"><xmp>
Read XML data file, transform and format display ...

ProcessingInstruction<xml>version="1.0" encoding="UTF-8"
Element<root>
        Element<bookstore>
                Element<book>
                        Attribute<ISBN>1-861003-11-0
                        Element<price>
                                Text<>8.99
                Element<book>
                        Attribute<ISBN>0-201-63361-2
                        Element<price>
                                Text<>11.99
                Element<book>
                        Attribute<ISBN>1-861001-57-6
                        Element<price>
                                Text<>9.99                                                             
</xmp></div>

<div class="indent" style="width:660">
As well as returning an XmlReader from the transformation, an XmlWriter can be supplied to the <b>Transform</b> method that enables the XML to be written to
a file or stream. The code sample below illustrates this, writing to a file called transform.xml, and then reading this to the console.
</div>

<div class="code"><xmp>
public void ReadTransformWrite()
{
    StreamReader stream = null;

    try
    {
        XmlDocument xmldocument = new XmlDocument();
        xmldocument.Load (m_Document);
        DocumentNavigator navigator = new DocumentNavigator(xmldocument);

        XslTransform xsltransform = new XslTransform();
        XmlTextWriter writer = new XmlTextWriter("transform.xml", null);

        xsltransform.Load(m_Stylesheet);
        xsltransform.Transform(navigator, null, writer);

        stream = new StreamReader ("transform.xml");
        Console.Write(stream.ReadToEnd());
    }

    catch (Exception e)
    {
        Console.WriteLine ("Exception: {0}", e.ToString());
    }

    finally
    {
        if (stream != null)
            stream.Close();
    }
}
</xmp></div>

<div class="indent" style="width:660">
The output from applying the same style sheet 
<a target="_blank" href="/quickstart/util/srcctrlwin.aspx?path=/quickstart/howto/samples/Xml/TransformXML/&file=books.xsl">books.xsl</a> is shown below.
</div>

<div class="code"><xmp>
Read XML data file, transform and write ...

<?xml version="1.0"?><root><bookstore><book ISBN="1-861003-11-0"><price>8.99</price>
                        </book><book ISBN="0-201-63361-2"><price>11.99</price>
                        </book><book ISBN="1-861001-57-6"><price>9.99</price>
                        </book></bookstore></root></xmp></div>

<H4>Summary</H4>
<OL>
<LI>XSLT is a language for transforming XML documents into other XML documents using XPath as the query language.
<LI>The XslTransform class Load method is used to load a stylesheet.	
<LI>The XslTransform class Transform method uses an XmlNavigator and either returns an XmlReader or writes the transformation into a supplied XmlWriter or stream.
<LI>XSL style sheets are combined to produce the required structure output format (e.g. XML or HTML).
<LI>XSLT engines do not manipulate documents, they manipulate structure.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
</xml>



















