<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/howto/doc/xml/header.inc" -->

<h4>How Do I...Apply validation when reading XML?</h4>

<div class="indent" style="width:660">
This sample illustrates how to apply validation when reading XML data using the XMLTextReader class. Validation is the process of enforcing rules on the XML
content either via a Document Type Definition (DTD) or a schema. At it's simplest, a schema is an XML representation of a DTD, but can also infer relationships 
within the XML data.<br clear="left"><br>
This sample follows on from the <a href="/quickstart/howto/doc/Xml/ReadXmlFile.aspx">How do I...Read XML from a file?</a> topic.
</div>

<h4>Applying validation when reading XML</h4> 
<Acme:SourceRef
ViewSource="/quickstart/howto/samples/Xml/ValidationReadingXml/ValidationReadingXML.src"
RunSample="/quickstart/howto/samples/Xml/ValidationReadingXml/ValidationReadingXML.aspx" 
Icon = "/quickstart/images/genicon.gif"
Caption="ValidationReadingXML.aspx"
runat="server" />

<br clear="left"><br>
<div class="indent" style="width:660">
In this sample we are going to apply validation when reading and parsing XML data using sample files which pass and fail.
The XmlTextReader can provide either DTD or XDR parse time validation. The <b>Validation</b> property can be used to request a particular validation option.
XDR is the Microsoft XML-Data Schema (which is a pre-cursor to the W3C XML schema) and is called the reduced set of XML Schema (also referred to as
XML-Data Reduced). This is described at <A target=_top href="http://www.ltg.ed.ac.uk/~ht/XMLData-Reduced.htm">XML-Data Reduced</A>.
The table below shows the types of validation which can be set.</div>

<br clear="left"><br>
<DIV class=indent>
<TABLE class=table 
style="border-style: solid" 
width="509">
<TBODY>
<TR>
	<TH width="179">Auto</TH>
	<TH width="308">This is the default.</TH>
</TR>
  <tr>
    <td width="179" height="17"><font size="1">Auto</font></td>
    <td width="308" height="17"><font size="1">This is the default.</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">None</font></td>
    <td width="308" height="19"><font size="1">No parse time validation.</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">DTD</font></td>
    <td width="308" height="19"><font size="1">Whether to validate according to a DTD.</font></td>
  </tr>
  <tr>
    <td width="179" height="19"><font size="1">Schema</font></td>
    <td width="509" height="19"><font size="1">Whether to validate according to an XDR schema.</font></td>
  </tr>
  </TBODY></TABLE></DIV>

<br clear="left"><br>
<div class="indent" style="width:660">
The Auto mode does the following:
<LI>If there is no DTD or schema then it will parse the XML without validation. 
<LI>If there is a DTD defined in a &lt;!DOCTYPE ...&gt; declaration then it will load the DTD and process the DTD declarations.
<LI>If there is no &lt;!DOCTYPE ...&gt; declaration but XDR schemas are specified (using the "x-schema:" prefix), then it loads and process those schemas. </LI></UL></DIV>

<div class="indent" style="width:660">
In the sample code 
<a href="/quickstart/util/srcview.aspx?path=/quickstart/howto/samples/Xml/ValidationReadingXml/ValidationReadingXML.src">View Source</a>
four files are validated. The first BooksDTD.xml validates against the books.dtd, the second BooksDTDFail.xml fails validation against the books.dtd,
the third BooksSchema.xml validates against the schema.xml file and the fourth BooksFailSchema.xml fails validation against the schema.xml.
Schema.xml is an XDR schema. To set the required type of validation, the <b>Validation</b> property is set either with the value of <b>DTD</b>
or <b>Schema</b> as illustrated below.</div>

<div class="code"><xmp>
XmlTextReader m_reader = new XmlTextReader (m_Document1);
m_reader.Validation = Validation.Schema;
m_reader.Validation = Validation.DTD;
</xmp></div>

<div class="indent" style="width:660">
Errors in validation result in the <b>ValidationCallback</b> method being called. This property is set with a <b>ValidationDelegate</b> class before reading the
XML document. If this is not provided then no validation errors are returned and parsing only stops if an XML well-formedness error occurs. However, if no
ValidationDelegate is provided you get default attributes and entities without having to deal with validation errors. The sample source code illustrates this.</div>

<div class="code"><xmp>
private void Validate()
{
    try
    {
        // Set the validation callback
        m_reader.ValidationCallback = new ValidationDelegate(this.ValidationCallback);
        // Read XML data
        while (m_reader.Read()){}
        Console.WriteLine ("Validation finished. Validation {0}", (m_success==true ? "successful" : "failed"));
    }
    catch (XmlException e)
    {
        Console.WriteLine ("XmlException: " + e.ToString());
    }

    catch (Exception e)
    {
        Console.WriteLine ("Exception: " + e.ToString());
    }
}

public void ValidationCallback (int errorid, String reason)
{
    m_success = false;
    Console.Write("\r\n\tValidation error: " + reason);
    if (m_reader.LineNumber > 0)
    {
        Console.WriteLine("Line: "+ m_reader.LineNumber + " Position: " + m_reader.LinePos);
    }
}
</xmp></div>

<div class="indent" style="width:660">
In this code sample, the class m_success variable indicates the validation state. The parser does not stop for any kind of validation error and only stops if
there is a well-formedness error during the read. This feature enables you find all the validation errors in one pass without having to continously re-parse
the XML.</div>

<br clear="left"><div class="indent" style="width:660">
The output from the sample is shown below, illustrating DTD and XDR schema successful and failed validations. If you examine the files you can 
determine the validation failures. e.g. The first book in the booksDTDFail.xml file does not have a title, which is required.</div>

<div class="code">
<xmp>
Validating XML file booksDTD.xml with DTD file books.dtd ...
Validation finished. Validation successful

Validation XML file booksDTDFail.xml with DTD file books.dtd ...

        Validation error: Invalid content. Expecting 'title'.Line: 6 Position: 12

        Validation error: Content is incomplete.Line: 12 Position: 2

        Validation error: Content is incomplete.Line: 12 Position: 2
Validation finished. Validation failed

Validation XML file booksSchema.xml with schema file schema.xml ...
Validation finished. Validation successful

Validation XML file booksSchemaFail.xml with schema file schema.xml ...

        Validation error: Missing required attribute 'genre'.Line: 4 Position: 4

        Validation error: Invalid content. Expecting 'title'.Line: 4 Position: 12

        Validation error: Content is incomplete.Line: 4 Position: 4

        Validation error: Content is incomplete.Line: 4 Position: 4
Validation finished. Validation failed
</xmp>
</div>

<H4>Summary</H4>
<OL>
<LI>Validation can either be against a DTD or an XDR schema, the type required being set by the Validation property.
<LI>Validation is performed during the read and parsing operations.
<LI>The ValidationCallback property must be set with a ValidationDelegate class to receive notification of validation errors.
<LI>Validation errors do not stop parsing. Parsing only stops if there is a well-formedness error. This enables all errors to be discovered in a single pass.
</LI></OL>

<!-- #include virtual="/quickstart/howto/include/footer.inc" -->









































