
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/include/header.inc" -->

<h4>ADO+: Filtering Data</h4>
<hr>
<p>
There are several ways to filter data. One way is to filter data at the database command level, by using a WHERE clause on your query. A second way is to filter the data after it is in the DataSet. This topic covers filtering in the DataSet.
<p>
Once data is in a DataSet you can use methods on the DataSet to get at subsets of that data. Consider a DataSet with Customers and Orders tables.  To filter on Data with Customers with FirstName is Kelly, you can use the Select method which returns an array of rows.
<P>
<div class=code>
<br> MyRowArray = dsCustomers.Select("ContactName like 'Kelly%'");
</div>
<p>
Notice that the statements are ANSI-SQL like. They are essentially a subset of ANSI-SQL, with the following difference: because the DataSet can hold multiple related tables, the filter can also filter over these related tables. In this example, we filter the orders as well as the customers by using the Child keyword:
<p>
<div class=code>
<br> MyRowArray = dsCustomers.Select("ContactName like 'Kelly%' AND [child].OrderDate = '7/26/68'");
</div>
<p>
These functions return an array of rows, through which you can iterate through using the For Each statement.
<p>
Another object, DataView, allows you to place multiple filters on a DataSet, set up databinding to them, and so forth. A DataView can be filtered using the same query language rules as in the Select, but the filter is dynamic. Thus, if a row is added to the data, and it qualifies with the filter, it is visible in the view. Views can be sorted and filtered. Filters include data filters and version filters (CURRENT, NEW, MODIFIED, DELETED). To set up a DataView, construct it by handing it a DataTable within a DataSet:
<p>
<div class=code>
<br> DataView dataView1 = new DataView(myDataSet.Tables["Customers"]);
</div>
<p>
You can set up a filter with the RowFilter property using the same query filter language as in the example above. To set up a sort, you can use a comma -delimited list of column sorts, followed by ASC (default and optional) or DESC for ascending and descending order.
<p>
<div class=code>
<br>	/// Sort the view based on the FirstName column
<br>	dataView1.Sort = "CustomerID";
<br>
<br>	// Filter the dataview to only show customers with the lastname = Smith
<br>	dataView1.RowFilter = "CustomerID='ALFKI'";
</div>
<p>
You can also filter based on the state of the rows (deleted, new, and so forth). Here is an example of a Filter on Deleted rows:
<p>
<div class=code>
<br> dataView1.RowStateFilter=DataViewRowState.Deleted;
</div>	
<p>
The sample below shows filling a DataSet, and then filtering it with a DataView. Note that a DataView can easily be databound. See the databinding topics in Win Forms or ASP+.
<p>
<Acme:SourceRef 
  RunSample="/quickstart/howto/samples/adoplus/cs/filterdatatable.aspx" 
  ViewSource="/quickstart/howto/samples/adoplus/cs/filterdatatable.src"
  Icon="/quickstart/images/genicon.gif"
  Caption="filterdatatable.aspx"
  runat="server" />
</a>
