
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/include/header.inc" -->

<h4>ADO+: Updating Data in a Database</h4>
<hr>
<p>
This topic illustrates how to update data in a database.  It is important to understand the topic <a href="getdatafromdb.aspx">Getting Data from a Database</a> before continuing.
<p>
Getting Data From a Database discusses retrieving data from a database and into a DataSet, and how the DataSet is separate and distinct from the database. Once the DataSet is loaded, you can modify the data, and the DataSet will track the changes.
<P>
Adding data is done through the Add method on the DataTable. The Add method takes either an array of the expected data columns, or a DataRow.
<p>
<div class=code>
<br>	dsCustomers = new DataSet();
<br>	DataRow Customer;
<br>
<br>	Customer = dsDataSet.Tables["Customers"].NewRow();
<br>	Customer["CustomerId"] ="NewID";
<br>	Customer["ContactName"] = "New Name";
<br>	Customer["CompanyName"] = "New Company Name";
<br>
<br>	dsCustomers.Tables["Customers"].Rows.Add(Customer);
</div>
<p>
Note that the DataTable must return a DataRow through the NewRow method. The method returns a DataRow object with the appropriate schema of the DataTable.  The new DataRow is independent of the table until it is added to the RowsCollection.
<p>
Changing data in a DataRow can be done by accessing the DataRow and changing the data. You can use the index of the row in the RowsCollection accessed through the Rows property:
<p>
<div class=code>
dsCustomers.Tables["Customers"].Rows[0]["ContactName"]="Peach";
</div>
<p>
You can also call the ByKey method on the table which returns a specific row by a key value:
<p>
<div class=code>
DataRow workRow = dsCustomers.Tables["Customers"].Rows.Find({"KeyValue"})
workRow["ContactName"]="Peach";
</div>
<p>
where Key is the Primary Key in the table. When using the SQLDataSetCommand, the Key is established from the Database. You can also set the Key if you are not using the database through the PrimaryKey property.
<P>
Removing the Row is done through the Delete method.  Note that you do not remove the row from the collection because this information must eventually be sent back to the database.  Similarly you can use RejectChanges on the DataSet, in which case the Row is restored.
<div class=code>
dsCustomers.Tables["Customers"].Rows[0].Delete;
</div>
<p>
The original and new values are maintained in the row. The RowChanging event allows you to access both original and new values to decide whether you want the edit to proceed. Because we maintain original and new values, we can establish scenarios such as optimistic locking and key changes.
<p>
To submit the Data from the DataSet into the database, use the Update method on the SQLDataSetCommand.
<p>
<div class=code>
CustomersDSCommand.Update(dsCustomers, "Customers");
</div>
<p>
The example below gets data from a Database using an SQLDataSetCommand, modifies the data within the DataSet, and then submits the data back to the database through the SQLDataSetCommand.
<p>
<Acme:SourceRef 
  RunSample="/quickstart/howto/samples/adoplus/cs/UpdatingData.aspx" 
  ViewSource="/quickstart/howto/samples/adoplus/cs/UpdatingData.src"
  Icon="/quickstart/images/genicon.gif"
  Caption="UpdatingData.aspx"
  runat="server" />
</a>
